import argparse
import re
import subprocess
import sys

import symuid


def _generate_uuid():
    return subprocess.check_output(['uuid', '-v', '4', '-F', 'BIN']).strip()


def _log_path(track_path, msg, stream=sys.stdout):
    stream.write("{!r}: {}\n".format(track_path, msg))


def _log_path_error(track_path, msg):
    _log_path(track_path, msg, stream=sys.stderr)


def sync(path, path_ignore_regex, show_ignored=False):
    for track in symuid.Track.walk(
            root_path=path,
            path_ignore_regex=path_ignore_regex,
            ignored_cb=lambda p: show_ignored and _log_path(p, 'ignored'),
            unsupported_cb=lambda p, e:
                _log_path_error(p, 'unsupported type, skipped'),
    ):
        if track.get_uuid() is None:
            track.assign_uuid(_generate_uuid())
            _log_path(track.path, 'assigned uuid {!r}'.format(
                track.get_uuid()))


def _init_argparser():
    argparser = argparse.ArgumentParser(description=None)
    argparser.add_argument('path')
    argparser.add_argument(
        '--path-ignore-regex',
        default=symuid.Track.PATH_DEFAULT_IGNORE_REGEX,
        nargs=1,
        metavar='pattern',
        dest='path_ignore_regex',
        type=re.compile,
        help='(default: %(default)s)',
    )
    argparser.add_argument(
        '--show-ignored',
        action='store_true',
    )
    return argparser


def _main():
    argparser = _init_argparser()
    args = argparser.parse_args()
    sync(**vars(args))
